==> mess_0.144.orig.tar.xz: mess-0.144/src/mess/machine/teleprinter.c <==
#include "emu.h"
#include "machine/teleprinter.h"
#include "machine/terminal.h"

/***************************************************************************
    TYPE DEFINITIONS
***************************************************************************/
#define TELEPRINTER_WIDTH 80
#define TELEPRINTER_HEIGHT 50
typedef struct _teleprinter_state teleprinter_state;
struct _teleprinter_state
{
	UINT8 buffer[TELEPRINTER_WIDTH*TELEPRINTER_HEIGHT];
	UINT8 x_pos;
	UINT8 last_code;
	UINT8 scan_line;
	devcb_resolved_write8 teleprinter_keyboard_func;
};

/*****************************************************************************
    INLINE FUNCTIONS
*****************************************************************************/
INLINE teleprinter_state *get_safe_token(device_t *device)
{
	assert(device != NULL);
	assert(device->type() == GENERIC_TELEPRINTER);

	return (teleprinter_state *)downcast<legacy_device_base *>(device)->token();
}

INLINE const teleprinter_interface *get_interface(device_t *device)
{
	assert(device != NULL);
	assert(device->type() == GENERIC_TELEPRINTER);
	return (const teleprinter_interface *) device->static_config();
}

/***************************************************************************
    IMPLEMENTATION
***************************************************************************/

static const UINT8 teleprinter_font[128*8] =
{
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x07,0x07,0x07,0x07,0x00,0x00,0x00,0x00,
  0x38,0x38,0x38,0x38,0x00,0x00,0x00,0x00,
  0x3f,0x3f,0x3f,0x3f,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x07,0x07,0x07,0x07,
  0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
  0x38,0x38,0x38,0x38,0x07,0x07,0x07,0x07,
  0x3f,0x3f,0x3f,0x3f,0x07,0x07,0x07,0x07,
  0x00,0x00,0x00,0x00,0x38,0x38,0x38,0x38,
  0x07,0x07,0x07,0x07,0x38,0x38,0x38,0x38,
  0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
  0x3f,0x3f,0x3f,0x3f,0x38,0x38,0x38,0x38,
  0x00,0x00,0x00,0x00,0x3f,0x3f,0x3f,0x3f,
  0x07,0x07,0x07,0x07,0x3f,0x3f,0x3f,0x3f,
  0x38,0x38,0x38,0x38,0x3f,0x3f,0x3f,0x3f,
  0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,
  0x00,0x00,0x00,0x00,0x2a,0x15,0x2a,0x15,
  0x2a,0x15,0x2a,0x15,0x00,0x00,0x00,0x00,
  0x3f,0x3f,0x3f,0x3f,0x2a,0x15,0x2a,0x15,
  0x2a,0x15,0x2a,0x15,0x3f,0x3f,0x3f,0x3f,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x08,0x08,0x08,0x08,0x08,0x00,0x08,0x00,
  0x14,0x14,0x14,0x00,0x00,0x00,0x00,0x00,
  0x14,0x14,0x3e,0x14,0x3e,0x14,0x14,0x00,
  0x08,0x3c,0x0a,0x1c,0x28,0x1e,0x08,0x00,
  0x06,0x26,0x10,0x08,0x04,0x32,0x30,0x00,
  0x08,0x14,0x14,0x0c,0x2a,0x12,0x2c,0x00,
  0x08,0x08,0x04,0x00,0x00,0x00,0x00,0x00,
  0x10,0x08,0x04,0x04,0x04,0x08,0x10,0x00,
  0x04,0x08,0x10,0x10,0x10,0x08,0x04,0x00,
  0x00,0x08,0x2a,0x1c,0x2a,0x08,0x00,0x00,
  0x00,0x08,0x08,0x3e,0x08,0x08,0x00,0x00,
  0x00,0x00,0x00,0x00,0x08,0x08,0x04,0x00,
  0x00,0x00,0x00,0x3e,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x00,
  0x00,0x20,0x10,0x08,0x04,0x02,0x00,0x00,
  0x1c,0x22,0x32,0x2a,0x26,0x22,0x1c,0x00,
  0x08,0x0c,0x08,0x08,0x08,0x08,0x1c,0x00,
  0x1c,0x22,0x20,0x18,0x04,0x02,0x3e,0x00,
  0x3e,0x20,0x10,0x18,0x20,0x22,0x1c,0x00,
  0x10,0x18,0x14,0x12,0x3e,0x10,0x10,0x00,
  0x3e,0x02,0x1e,0x20,0x20,0x22,0x1c,0x00,
  0x10,0x08,0x04,0x1c,0x22,0x22,0x1c,0x00,
  0x3e,0x20,0x10,0x08,0x04,0x04,0x04,0x00,
  0x1c,0x22,0x22,0x1c,0x22,0x22,0x1c,0x00,
  0x1c,0x22,0x22,0x1c,0x10,0x08,0x04,0x00,
  0x00,0x00,0x00,0x08,0x00,0x08,0x00,0x00,
  0x00,0x00,0x08,0x00,0x08,0x08,0x04,0x00,
  0x10,0x08,0x04,0x02,0x04,0x08,0x10,0x00,
  0x00,0x00,0x3e,0x00,0x3e,0x00,0x00,0x00,
  0x04,0x08,0x10,0x20,0x10,0x08,0x04,0x00,
  0x1c,0x22,0x20,0x10,0x08,0x00,0x08,0x00,
  0x1c,0x22,0x32,0x2a,0x3a,0x02,0x3c,0x00,
  0x08,0x14,0x22,0x22,0x3e,0x22,0x22,0x00,
  0x1e,0x22,0x22,0x1e,0x22,0x22,0x1e,0x00,
  0x1c,0x22,0x02,0x02,0x02,0x22,0x1c,0x00,
  0x1e,0x24,0x24,0x24,0x24,0x24,0x1e,0x00,
  0x3e,0x02,0x02,0x1e,0x02,0x02,0x3e,0x00,
  0x3e,0x02,0x02,0x1e,0x02,0x02,0x02,0x00,
  0x1c,0x22,0x02,0x02,0x32,0x22,0x3c,0x00,
  0x22,0x22,0x22,0x3e,0x22,0x22,0x22,0x00,
  0x1c,0x08,0x08,0x08,0x08,0x08,0x1c,0x00,
  0x38,0x10,0x10,0x10,0x10,0x12,0x0c,0x00,
  0x22,0x12,0x0a,0x06,0x0a,0x12,0x22,0x00,
  0x02,0x02,0x02,0x02,0x02,0x02,0x3e,0x00,
  0x22,0x36,0x2a,0x2a,0x22,0x22,0x22,0x00,
  0x22,0x22,0x26,0x2a,0x32,0x22,0x22,0x00,
  0x1c,0x22,0x22,0x22,0x22,0x22,0x1c,0x00,
  0x1e,0x22,0x22,0x1e,0x02,0x02,0x02,0x00,
  0x1c,0x22,0x22,0x22,0x2a,0x12,0x2c,0x00,
  0x1e,0x22,0x22,0x1e,0x0a,0x12,0x22,0x00,
  0x1c,0x22,0x02,0x1c,0x20,0x22,0x1c,0x00,
  0x3e,0x08,0x08,0x08,0x08,0x08,0x08,0x00,
  0x22,0x22,0x22,0x22,0x22,0x22,0x1c,0x00,
  0x22,0x22,0x22,0x14,0x14,0x08,0x08,0x00,
  0x22,0x22,0x22,0x2a,0x2a,0x2a,0x14,0x00,
  0x22,0x22,0x14,0x08,0x14,0x22,0x22,0x00,
  0x22,0x22,0x22,0x14,0x08,0x08,0x08,0x00,
  0x3e,0x20,0x10,0x08,0x04,0x02,0x3e,0x00,
  0x0e,0x02,0x02,0x02,0x02,0x02,0x0e,0x00,
  0x00,0x02,0x04,0x08,0x10,0x20,0x00,0x00,
  0x38,0x20,0x20,0x20,0x20,0x20,0x38,0x00,
  0x08,0x1c,0x2a,0x08,0x08,0x08,0x08,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0x00,
  0x04,0x08,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x1c,0x20,0x3c,0x22,0x3c,0x00,
  0x02,0x02,0x1e,0x22,0x22,0x22,0x1e,0x00,
  0x00,0x00,0x3c,0x02,0x02,0x02,0x3c,0x00,
  0x20,0x20,0x3c,0x22,0x22,0x22,0x3c,0x00,
  0x00,0x00,0x1c,0x22,0x3e,0x02,0x1c,0x00,
  0x18,0x04,0x0e,0x04,0x04,0x04,0x04,0x00,
  0x00,0x00,0x3c,0x22,0x22,0x3c,0x20,0x18,
  0x02,0x02,0x1e,0x22,0x22,0x22,0x22,0x00,
  0x08,0x00,0x0c,0x08,0x08,0x08,0x1c,0x00,
  0x10,0x00,0x18,0x10,0x10,0x10,0x12,0x0c,
  0x02,0x02,0x22,0x12,0x0e,0x16,0x22,0x00,
  0x08,0x08,0x08,0x08,0x08,0x08,0x10,0x00,
  0x00,0x00,0x16,0x2a,0x2a,0x2a,0x2a,0x00,
  0x00,0x00,0x1a,0x26,0x22,0x22,0x22,0x00,
  0x00,0x00,0x1c,0x22,0x22,0x22,0x1c,0x00,
  0x00,0x00,0x1e,0x22,0x22,0x1e,0x02,0x02,
  0x00,0x00,0x3c,0x22,0x22,0x3c,0x20,0x20,
  0x00,0x00,0x34,0x0c,0x04,0x04,0x04,0x00,
  0x00,0x00,0x3c,0x02,0x1c,0x20,0x1e,0x00,
  0x08,0x08,0x1c,0x08,0x08,0x08,0x10,0x00,
  0x00,0x00,0x22,0x22,0x22,0x32,0x2c,0x00,
  0x00,0x00,0x22,0x22,0x22,0x14,0x08,0x00,
  0x00,0x00,0x22,0x22,0x2a,0x2a,0x14,0x00,
  0x00,0x00,0x22,0x14,0x08,0x14,0x22,0x00,
  0x00,0x00,0x22,0x22,0x14,0x08,0x04,0x02,
  0x00,0x00,0x3e,0x10,0x08,0x04,0x3e,0x00,
  0x10,0x08,0x08,0x04,0x08,0x08,0x10,0x00,
  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x00,
  0x04,0x08,0x08,0x10,0x08,0x08,0x04,0x00,
  0x3e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x15,0x2a,0x15,0x2a,0x15,0x2a,0x15
};

static void teleprinter_scroll_line(device_t *device)
{
	teleprinter_state *term = get_safe_token(device);

	memcpy(term->buffer,term->buffer+TELEPRINTER_WIDTH,(TELEPRINTER_HEIGHT-1)*TELEPRINTER_WIDTH);
	memset(term->buffer + TELEPRINTER_WIDTH*(TELEPRINTER_HEIGHT-1),0x20,TELEPRINTER_WIDTH);
}

static void teleprinter_write_char(device_t *device,UINT8 data) {
	teleprinter_state *term = get_safe_token(device);

	term->buffer[(TELEPRINTER_HEIGHT-1)*TELEPRINTER_WIDTH+term->x_pos] = data;
	term->x_pos++;
	if (term->x_pos >= TELEPRINTER_WIDTH)
	{
		term->x_pos = 0;
		teleprinter_scroll_line(device);
	}
}

static void teleprinter_clear(device_t *device) {
	teleprinter_state *term = get_safe_token(device);

	memset(term->buffer,0,TELEPRINTER_WIDTH*TELEPRINTER_HEIGHT);
	term->x_pos = 0;
}

WRITE8_DEVICE_HANDLER ( teleprinter_write )
{
	teleprinter_state *term = get_safe_token(device);
	switch(data) {
		case 10: term->x_pos = 0;
				teleprinter_scroll_line(device);
				break;
		case 13: term->x_pos = 0; break;
		case  9: term->x_pos = (term->x_pos & 0xf8) + 8;
				if (term->x_pos >= TELEPRINTER_WIDTH)
					term->x_pos = TELEPRINTER_WIDTH-1;

				break;
		case 16: break;
		default: teleprinter_write_char(device,data); break;
	}
}

/***************************************************************************
    VIDEO HARDWARE
***************************************************************************/
static void generic_teleprinter_update(device_t *device, bitmap_t *bitmap, const rectangle *cliprect)
{
	UINT8 code;
	int y, c, x, b;
	teleprinter_state *term = get_safe_token(device);

	for (y = 0; y < TELEPRINTER_HEIGHT; y++)
	{
		for (c = 0; c < 8; c++)
		{
			int horpos = 0;
			for (x = 0; x < TELEPRINTER_WIDTH; x++)
			{
				code = teleprinter_font[(term->buffer[y*TELEPRINTER_WIDTH + x]  & 0x7f) *8 + c];
				for (b = 0; b < 8; b++)
				{
					*BITMAP_ADDR16(bitmap, y*8 + c, horpos++) =  (code >> b) & 0x01 ? 0 : 1;
				}
			}
		}
	}
}

static TIMER_CALLBACK(keyboard_callback)
{
	teleprinter_state *term = get_safe_token((device_t *)ptr);
	term->last_code = terminal_keyboard_handler(machine, term->teleprinter_keyboard_func, term->last_code, &term->scan_line, NULL, NULL, (device_t *)ptr);
}

/***************************************************************************
    VIDEO HARDWARE
***************************************************************************/
static VIDEO_START( teleprinter )
{

}

static SCREEN_UPDATE(teleprinter )
{
	device_t *devconf = screen->machine().device(TELEPRINTER_TAG);
	generic_teleprinter_update( devconf, bitmap, cliprect);
	return 0;
}

MACHINE_CONFIG_FRAGMENT( generic_teleprinter )
	MCFG_SCREEN_ADD(TELEPRINTER_SCREEN_TAG, RASTER)
	MCFG_SCREEN_REFRESH_RATE(50)
	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
	MCFG_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
	MCFG_SCREEN_SIZE(TELEPRINTER_WIDTH*8, TELEPRINTER_HEIGHT*8)
	MCFG_SCREEN_VISIBLE_AREA(0, TELEPRINTER_WIDTH*8-1, 0, TELEPRINTER_HEIGHT*8-1)
	MCFG_SCREEN_UPDATE(teleprinter)

	MCFG_PALETTE_LENGTH(2)
	MCFG_PALETTE_INIT(black_and_white)

	MCFG_VIDEO_START(teleprinter)
MACHINE_CONFIG_END


/*-------------------------------------------------
    DEVICE_START( teleprinter )
-------------------------------------------------*/

static DEVICE_START( teleprinter )
{
	teleprinter_state *term = get_safe_token(device);
	const teleprinter_interface *intf = get_interface(device);

	term->teleprinter_keyboard_func.resolve(intf->teleprinter_keyboard_func, *device);

	if (!term->teleprinter_keyboard_func.isnull())
		device->machine().scheduler().timer_pulse(attotime::from_hz(240), FUNC(keyboard_callback), 0, (void*)device);
}

/*-------------------------------------------------
    DEVICE_RESET( teleprinter )
-------------------------------------------------*/

static DEVICE_RESET( teleprinter )
{
	teleprinter_state *term = get_safe_token(device);

	teleprinter_clear(device);
	term->last_code = 0;
	term->scan_line = 0;
}

/*-------------------------------------------------
    DEVICE_GET_INFO( teleprinter )
-------------------------------------------------*/

DEVICE_GET_INFO( teleprinter )
{
	switch (state)
	{
		/* --- the following bits of info are returned as 64-bit signed integers --- */
		case DEVINFO_INT_INLINE_CONFIG_BYTES:			info->i = 0;												break;
		case DEVINFO_INT_TOKEN_BYTES:					info->i = sizeof(teleprinter_state);							break;

		/* --- the following bits of info are returned as pointers to data or functions --- */
		case DEVINFO_FCT_START:							info->start = DEVICE_START_NAME(teleprinter);					break;
		case DEVINFO_FCT_STOP:							/* Nothing */												break;
		case DEVINFO_FCT_RESET:							info->reset = DEVICE_RESET_NAME(teleprinter);					break;

		case DEVINFO_PTR_INPUT_PORTS:					info->ipt = INPUT_PORTS_NAME(generic_terminal);				break;
		/* --- the following bits of info are returned as NULL-terminated strings --- */
		case DEVINFO_STR_NAME:							strcpy(info->s, "Generic Teleprinter");						break;
		case DEVINFO_STR_FAMILY:						strcpy(info->s, "Teleprinter");								break;
		case DEVINFO_STR_VERSION:						strcpy(info->s, "1.0");										break;
		case DEVINFO_STR_SOURCE_FILE:					strcpy(info->s, __FILE__);									break;
		case DEVINFO_STR_CREDITS:						strcpy(info->s, "Copyright the MESS Team"); 				break;
	}
}

DEFINE_LEGACY_DEVICE(GENERIC_TELEPRINTER, teleprinter);

==> mess_0.146.orig.tar.xz: mess-0.146/src/mess/machine/teleprinter.c <==
#include "machine/teleprinter.h"

static const UINT8 teleprinter_font[128*8] =
{
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x07,0x07,0x07,0x07,0x00,0x00,0x00,0x00,
  0x38,0x38,0x38,0x38,0x00,0x00,0x00,0x00,
  0x3f,0x3f,0x3f,0x3f,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x07,0x07,0x07,0x07,
  0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,
  0x38,0x38,0x38,0x38,0x07,0x07,0x07,0x07,
  0x3f,0x3f,0x3f,0x3f,0x07,0x07,0x07,0x07,
  0x00,0x00,0x00,0x00,0x38,0x38,0x38,0x38,
  0x07,0x07,0x07,0x07,0x38,0x38,0x38,0x38,
  0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
  0x3f,0x3f,0x3f,0x3f,0x38,0x38,0x38,0x38,
  0x00,0x00,0x00,0x00,0x3f,0x3f,0x3f,0x3f,
  0x07,0x07,0x07,0x07,0x3f,0x3f,0x3f,0x3f,
  0x38,0x38,0x38,0x38,0x3f,0x3f,0x3f,0x3f,
  0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,
  0x00,0x00,0x00,0x00,0x2a,0x15,0x2a,0x15,
  0x2a,0x15,0x2a,0x15,0x00,0x00,0x00,0x00,
  0x3f,0x3f,0x3f,0x3f,0x2a,0x15,0x2a,0x15,
  0x2a,0x15,0x2a,0x15,0x3f,0x3f,0x3f,0x3f,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x08,0x08,0x08,0x08,0x08,0x00,0x08,0x00,
  0x14,0x14,0x14,0x00,0x00,0x00,0x00,0x00,
  0x14,0x14,0x3e,0x14,0x3e,0x14,0x14,0x00,
  0x08,0x3c,0x0a,0x1c,0x28,0x1e,0x08,0x00,
  0x06,0x26,0x10,0x08,0x04,0x32,0x30,0x00,
  0x08,0x14,0x14,0x0c,0x2a,0x12,0x2c,0x00,
  0x08,0x08,0x04,0x00,0x00,0x00,0x00,0x00,
  0x10,0x08,0x04,0x04,0x04,0x08,0x10,0x00,
  0x04,0x08,0x10,0x10,0x10,0x08,0x04,0x00,
  0x00,0x08,0x2a,0x1c,0x2a,0x08,0x00,0x00,
  0x00,0x08,0x08,0x3e,0x08,0x08,0x00,0x00,
  0x00,0x00,0x00,0x00,0x08,0x08,0x04,0x00,
  0x00,0x00,0x00,0x3e,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x00,
  0x00,0x20,0x10,0x08,0x04,0x02,0x00,0x00,
  0x1c,0x22,0x32,0x2a,0x26,0x22,0x1c,0x00,
  0x08,0x0c,0x08,0x08,0x08,0x08,0x1c,0x00,
  0x1c,0x22,0x20,0x18,0x04,0x02,0x3e,0x00,
  0x3e,0x20,0x10,0x18,0x20,0x22,0x1c,0x00,
  0x10,0x18,0x14,0x12,0x3e,0x10,0x10,0x00,
  0x3e,0x02,0x1e,0x20,0x20,0x22,0x1c,0x00,
  0x10,0x08,0x04,0x1c,0x22,0x22,0x1c,0x00,
  0x3e,0x20,0x10,0x08,0x04,0x04,0x04,0x00,
  0x1c,0x22,0x22,0x1c,0x22,0x22,0x1c,0x00,
  0x1c,0x22,0x22,0x1c,0x10,0x08,0x04,0x00,
  0x00,0x00,0x00,0x08,0x00,0x08,0x00,0x00,
  0x00,0x00,0x08,0x00,0x08,0x08,0x04,0x00,
  0x10,0x08,0x04,0x02,0x04,0x08,0x10,0x00,
  0x00,0x00,0x3e,0x00,0x3e,0x00,0x00,0x00,
  0x04,0x08,0x10,0x20,0x10,0x08,0x04,0x00,
  0x1c,0x22,0x20,0x10,0x08,0x00,0x08,0x00,
  0x1c,0x22,0x32,0x2a,0x3a,0x02,0x3c,0x00,
  0x08,0x14,0x22,0x22,0x3e,0x22,0x22,0x00,
  0x1e,0x22,0x22,0x1e,0x22,0x22,0x1e,0x00,
  0x1c,0x22,0x02,0x02,0x02,0x22,0x1c,0x00,
  0x1e,0x24,0x24,0x24,0x24,0x24,0x1e,0x00,
  0x3e,0x02,0x02,0x1e,0x02,0x02,0x3e,0x00,
  0x3e,0x02,0x02,0x1e,0x02,0x02,0x02,0x00,
  0x1c,0x22,0x02,0x02,0x32,0x22,0x3c,0x00,
  0x22,0x22,0x22,0x3e,0x22,0x22,0x22,0x00,
  0x1c,0x08,0x08,0x08,0x08,0x08,0x1c,0x00,
  0x38,0x10,0x10,0x10,0x10,0x12,0x0c,0x00,
  0x22,0x12,0x0a,0x06,0x0a,0x12,0x22,0x00,
  0x02,0x02,0x02,0x02,0x02,0x02,0x3e,0x00,
  0x22,0x36,0x2a,0x2a,0x22,0x22,0x22,0x00,
  0x22,0x22,0x26,0x2a,0x32,0x22,0x22,0x00,
  0x1c,0x22,0x22,0x22,0x22,0x22,0x1c,0x00,
  0x1e,0x22,0x22,0x1e,0x02,0x02,0x02,0x00,
  0x1c,0x22,0x22,0x22,0x2a,0x12,0x2c,0x00,
  0x1e,0x22,0x22,0x1e,0x0a,0x12,0x22,0x00,
  0x1c,0x22,0x02,0x1c,0x20,0x22,0x1c,0x00,
  0x3e,0x08,0x08,0x08,0x08,0x08,0x08,0x00,
  0x22,0x22,0x22,0x22,0x22,0x22,0x1c,0x00,
  0x22,0x22,0x22,0x14,0x14,0x08,0x08,0x00,
  0x22,0x22,0x22,0x2a,0x2a,0x2a,0x14,0x00,
  0x22,0x22,0x14,0x08,0x14,0x22,0x22,0x00,
  0x22,0x22,0x22,0x14,0x08,0x08,0x08,0x00,
  0x3e,0x20,0x10,0x08,0x04,0x02,0x3e,0x00,
  0x0e,0x02,0x02,0x02,0x02,0x02,0x0e,0x00,
  0x00,0x02,0x04,0x08,0x10,0x20,0x00,0x00,
  0x38,0x20,0x20,0x20,0x20,0x20,0x38,0x00,
  0x08,0x1c,0x2a,0x08,0x08,0x08,0x08,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0x00,
  0x04,0x08,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x1c,0x20,0x3c,0x22,0x3c,0x00,
  0x02,0x02,0x1e,0x22,0x22,0x22,0x1e,0x00,
  0x00,0x00,0x3c,0x02,0x02,0x02,0x3c,0x00,
  0x20,0x20,0x3c,0x22,0x22,0x22,0x3c,0x00,
  0x00,0x00,0x1c,0x22,0x3e,0x02,0x1c,0x00,
  0x18,0x04,0x0e,0x04,0x04,0x04,0x04,0x00,
  0x00,0x00,0x3c,0x22,0x22,0x3c,0x20,0x18,
  0x02,0x02,0x1e,0x22,0x22,0x22,0x22,0x00,
  0x08,0x00,0x0c,0x08,0x08,0x08,0x1c,0x00,
  0x10,0x00,0x18,0x10,0x10,0x10,0x12,0x0c,
  0x02,0x02,0x22,0x12,0x0e,0x16,0x22,0x00,
  0x08,0x08,0x08,0x08,0x08,0x08,0x10,0x00,
  0x00,0x00,0x16,0x2a,0x2a,0x2a,0x2a,0x00,
  0x00,0x00,0x1a,0x26,0x22,0x22,0x22,0x00,
  0x00,0x00,0x1c,0x22,0x22,0x22,0x1c,0x00,
  0x00,0x00,0x1e,0x22,0x22,0x1e,0x02,0x02,
  0x00,0x00,0x3c,0x22,0x22,0x3c,0x20,0x20,
  0x00,0x00,0x34,0x0c,0x04,0x04,0x04,0x00,
  0x00,0x00,0x3c,0x02,0x1c,0x20,0x1e,0x00,
  0x08,0x08,0x1c,0x08,0x08,0x08,0x10,0x00,
  0x00,0x00,0x22,0x22,0x22,0x32,0x2c,0x00,
  0x00,0x00,0x22,0x22,0x22,0x14,0x08,0x00,
  0x00,0x00,0x22,0x22,0x2a,0x2a,0x14,0x00,
  0x00,0x00,0x22,0x14,0x08,0x14,0x22,0x00,
  0x00,0x00,0x22,0x22,0x14,0x08,0x04,0x02,
  0x00,0x00,0x3e,0x10,0x08,0x04,0x3e,0x00,
  0x10,0x08,0x08,0x04,0x08,0x08,0x10,0x00,
  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x00,
  0x04,0x08,0x08,0x10,0x08,0x08,0x04,0x00,
  0x3e,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x15,0x2a,0x15,0x2a,0x15,0x2a,0x15
};

teleprinter_device::teleprinter_device(const machine_config &mconfig, const char *tag, device_t *owner, UINT32 clock)
	: generic_terminal_device(mconfig, TELEPRINTER, "Teleprinter", tag, owner, clock)
{
}

void teleprinter_device::scroll_line()
{
	memcpy(m_buffer,m_buffer+TELEPRINTER_WIDTH,(TELEPRINTER_HEIGHT-1)*TELEPRINTER_WIDTH);
	memset(m_buffer + TELEPRINTER_WIDTH*(TELEPRINTER_HEIGHT-1),0x20,TELEPRINTER_WIDTH);
}

void teleprinter_device::write_char(UINT8 data) {
	m_buffer[(TELEPRINTER_HEIGHT-1)*TELEPRINTER_WIDTH+m_x_pos] = data;
	m_x_pos++;
	if (m_x_pos >= TELEPRINTER_WIDTH)
	{
		m_x_pos = 0;
		scroll_line();
	}
}

void teleprinter_device::clear() {
	memset(m_buffer,0,TELEPRINTER_WIDTH*TELEPRINTER_HEIGHT);
	m_x_pos = 0;
}

void teleprinter_device::term_write(UINT8 data)
{
	switch(data) {
		case 10: m_x_pos = 0;
				scroll_line();
				break;
		case 13: m_x_pos = 0; break;
		case  9: m_x_pos = (m_x_pos & 0xf8) + 8;
				if (m_x_pos >= TELEPRINTER_WIDTH)
					m_x_pos = TELEPRINTER_WIDTH-1;

				break;
		case 16: break;
		default: write_char(data); break;
	}
}

/***************************************************************************
    VIDEO HARDWARE
***************************************************************************/
UINT32 teleprinter_device::tp_update(screen_device &device, bitmap_rgb32 &bitmap, const rectangle &cliprect)
{
	UINT8 code;
	int y, c, x, b;

	for (y = 0; y < TELEPRINTER_HEIGHT; y++)
	{
		for (c = 0; c < 8; c++)
		{
			int horpos = 0;
			for (x = 0; x < TELEPRINTER_WIDTH; x++)
			{
				code = teleprinter_font[(m_buffer[y*TELEPRINTER_WIDTH + x]  & 0x7f) *8 + c];
				for (b = 0; b < 8; b++)
				{
					bitmap.pix32(y*8 + c, horpos++) =  (code >> b) & 0x01 ? 0 : 0x00ffffff;
				}
			}
		}
	}
	return 0;
}

/***************************************************************************
    VIDEO HARDWARE
***************************************************************************/
MACHINE_CONFIG_FRAGMENT( generic_teleprinter )
	MCFG_SCREEN_ADD(TELEPRINTER_SCREEN_TAG, RASTER)
	MCFG_SCREEN_REFRESH_RATE(50)
	MCFG_SCREEN_VBLANK_TIME(ATTOSECONDS_IN_USEC(2500)) /* not accurate */
	MCFG_SCREEN_SIZE(TELEPRINTER_WIDTH*8, TELEPRINTER_HEIGHT*8)
	MCFG_SCREEN_VISIBLE_AREA(0, TELEPRINTER_WIDTH*8-1, 0, TELEPRINTER_HEIGHT*8-1)
	MCFG_SCREEN_UPDATE_DEVICE(DEVICE_SELF, teleprinter_device, tp_update)
MACHINE_CONFIG_END

machine_config_constructor teleprinter_device::device_mconfig_additions() const
{
      return MACHINE_CONFIG_NAME(generic_teleprinter);
}

/*-------------------------------------------------
    DEVICE_RESET( teleprinter )
-------------------------------------------------*/

void teleprinter_device::device_reset()
{
	clear();
	generic_terminal_device::device_reset();
}

const device_type TELEPRINTER = &device_creator<teleprinter_device>;
